package com.primerEjemplo.Modelo;

import com.github.javafaker.Faker;

import java.io.File;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Objects;

public class Libro {

    private final String isbn;
    private final String titulo;
    private final long numeroPaginas;
    private final String fechaPublicacion;
    private final String url;
    private final String descripcionCorta;
    private final String descripcionLarga;
    private final String estado;
    private final List<String> autores;
    private final List<String> categorias;

    private String anyo;
    private LocalDate localDate;

    public Libro(String isbn, String titulo, long numeroPaginas, String fechaPublicacion, String url, String descripcionCorta, String descripcionLarga, String estado, List<String> autores, List<String> categorias) {
        this.isbn = isbn;
        this.titulo = titulo;
        this.numeroPaginas = numeroPaginas;
        this.fechaPublicacion = conversionFecha(fechaPublicacion);
        this.url = url;
        this.descripcionCorta = descripcionCorta;
        this.descripcionLarga = descripcionLarga;
        this.estado = estado;
        this.autores = autores;
        this.categorias = categorias;
    }

    public String getIsbn() {
        return isbn;
    }

    public String getTitulo() {
        return titulo;
    }

    public long getNumeroPaginas() {
        return numeroPaginas;
    }

    public String getFechaPublicacion() {
        return fechaPublicacion;
    }

    public String getUrl() {
        return url;
    }

    public String getDescripcionCorta() {
        return descripcionCorta;
    }

    public String getDescripcionLarga() {
        return descripcionLarga;
    }

    public String getEstado() {
        return estado;
    }

    public List<String> getAutores() {
        return autores;
    }

    public List<String> getCategorias() {
        return categorias;
    }

    public String conversionFecha(String fecha) {
        if (fecha == null) {
            return generarFecha();
        }
        DateTimeFormatter inputFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd'T'HH:mm:ss.SSS");
        DateTimeFormatter outputFormatter = DateTimeFormatter.ofPattern("dd-MM-yyy");
        localDate = LocalDate.parse(fecha, inputFormatter);
        anyo = String.valueOf(localDate.getYear());
        return outputFormatter.format(localDate);

    }

    public String getAnyo(){
        if (anyo == null){
            anyo = String.valueOf(localDate.getYear());
        }
        return anyo;
    }

    public String generarFecha(){
        Faker faker = new Faker();
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.MONTH,11);
        calendar.set(Calendar.DATE, 5);
        calendar.set(Calendar.YEAR,2009);
        Calendar calendar1 = Calendar.getInstance();
        calendar1.set(Calendar.MONTH,11);
        calendar1.set(Calendar.DATE, 5);
        calendar1.set(Calendar.YEAR,2009);
        Date date = faker.date().between(calendar.getTime(),calendar1.getTime());
        SimpleDateFormat ft = new SimpleDateFormat ("yyyy-MM-dd");
        String fechaFormateada = ft.format(date);
        localDate = LocalDate.parse(fechaFormateada);
        anyo = String.valueOf(localDate.getYear());
        return fechaFormateada;
    }




    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        Libro libro = (Libro) o;
        return Objects.equals(isbn, libro.isbn) && Objects.equals(fechaPublicacion, libro.fechaPublicacion) && Objects.equals(estado, libro.estado) && Objects.equals(categorias, libro.categorias);
    }

    @Override
    public int hashCode() {
        return Objects.hash(isbn, fechaPublicacion, estado, categorias);
    }

    public String printarResumenLibro(){
        return "Libro{" +
                "isbn='" + isbn + '\'' +
                ", titulo='" + titulo + '\'' +
                ", numeroPaginas=" + numeroPaginas +
                ", fechaPublicacion='" + fechaPublicacion + '\'' +
                ", estado='" + estado + '\'' +
                ", autores=" + autores +
                ", categorias='" + categorias + '\'' +
                '}';
    }


    @Override
    public String toString() {
        return "Libro{" +
                "isbn='" + isbn + '\'' +
                ", titulo='" + titulo + '\'' +
                ", numeroPaginas=" + numeroPaginas +
                ", fechaPublicacion='" + fechaPublicacion + '\'' +
                ", url='" + url + '\'' +
                ", descripcionCorta='" + descripcionCorta + '\'' +
                ", descripcionLarga='" + descripcionLarga + '\'' +
                ", estado='" + estado + '\'' +
                ", autores=" + autores +
                ", categorias='" + categorias + '\'' +
                '}';
    }
}
