package com.primerEjemplo.Modelo;


import com.github.javafaker.Faker;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.json.JSONTokener;
import java.io.*;
import java.nio.charset.StandardCharsets;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

public class ParserLibros {

    private List<Libro> libros;
    private final InputStream inputStream;

    private Faker faker = new Faker();


    public ParserLibros(File file) throws FileNotFoundException {
        inputStream = new FileInputStream(file);
    }

    public boolean startParser() {
        List<String> autores;
        List<String> categorias;


        libros = new ArrayList<>();

        try {
            int size = inputStream.available();
            byte[] buffer = new byte[size];
            inputStream.read(buffer);
            inputStream.close();
            String json = new String(buffer, StandardCharsets.UTF_8);
            JSONTokener jsonTokener = new JSONTokener(json);
            JSONArray jsonArray = new JSONArray(jsonTokener);
            libros = new ArrayList<>();
            for (int i = 0; i < jsonArray.length(); i++) {
                JSONObject jsonObject = jsonArray.getJSONObject(i);
                String titulo = jsonObject.getString("title");
                long numeroPaginas = jsonObject.getLong("pageCount");
                String estado = jsonObject.getString("status");
                //Variables que no pueden estar
                String isbn;
                String descripcionCorta;
                String descripcionLarga;
                String url;
                String fechaPublicacion;
                try {
                    isbn = jsonObject.getString("isbn");
                    descripcionCorta = jsonObject.getString("shortDescription");
                    descripcionLarga = jsonObject.getString("longDescription");
                    url = jsonObject.getString("thumbnailUrl");
                    fechaPublicacion = jsonObject.getString("publishedDate");
                } catch (JSONException j) {
                    descripcionCorta = null;
                    descripcionLarga = null;
                    url = null;
                    isbn =  faker.number().digits(10);
                    fechaPublicacion = null;
                }

                //Rellenamos la matriz de autores
                JSONArray matrizAutores = jsonObject.getJSONArray("authors");
                autores = new ArrayList<>(matrizAutores.length());
                for (int j = 0; j < matrizAutores.length(); j++) {
                    String autoresRecogidos = matrizAutores.getString(j);
                    autores.add(autoresRecogidos);
                }
                //Rellenamos la matriz de categorias
                JSONArray matrizCategorias = jsonObject.getJSONArray("categories");
                categorias = new ArrayList<>(matrizCategorias.length());
                for (int j = 0; j < matrizCategorias.length(); j++) {
                    String categoriasRecogidas = matrizCategorias.getString(j);
                    categorias.add(categoriasRecogidas);
                }
                String fechaFormateada = null;
                if (fechaPublicacion != null) {
                    fechaFormateada = fechaPublicacion.substring(0, 23);
                }
                libros.add(new Libro(isbn, titulo, numeroPaginas, fechaFormateada, url, descripcionCorta, descripcionLarga, estado, autores, categorias));
            }
            return true;

        } catch (IOException e) {
            e.printStackTrace();
        }

        return false;
    }





    public List<Libro> getLibros() {
        return libros;
    }
}
