package com.primerEjemplo.Modelo.Servicio;

import com.primerEjemplo.Modelo.Interfaz.IServicioConsulta;
import com.primerEjemplo.Modelo.Interfaz.IServicioLibro;
import com.primerEjemplo.Modelo.Libro;
import org.springframework.beans.factory.annotation.Autowired;

import java.time.LocalDate;
import java.util.Arrays;
import java.util.List;
import java.util.function.Predicate;


public class ImplementaIntefazServicioConsultas implements IServicioConsulta {

    Predicate<Libro> libroPredicate;

    @Autowired
    IServicioLibro iServicioLibro;

    public ImplementaIntefazServicioConsultas()  {
        libroPredicate = null;
    }

    @Override
    public List<Libro> ejecutarLibro() {
//        return iServicioLibro.mostrarLibros().stream().filter(libroPredicate).toList();
        return iServicioLibro.mostrarLibrosResumen().stream().filter(libroPredicate).toList();
    }

    /**
     *
     * @param categorias busca la primera categoria que encuentra similar
     * @return la interfaz recargada con la nueva logica dentro del predicate que servira al ejecutar el libro.
     */
    @Override
    public IServicioConsulta cualquierCategoria(String... categorias) {
        Predicate<Libro> cualquierGenero = libro -> Arrays.stream(categorias).anyMatch(libro.getCategorias()::contains);
        libroPredicate = (libroPredicate == null) ? cualquierGenero : libroPredicate.and(cualquierGenero);
        return this;
    }

    /**
     *
     * @param categorias busca todas las categorias que encuentra similar
     * @return la interfaz recargada con la nueva logica dentro del predicate que servira al ejecutar el libro.
     */
    @Override
    public IServicioConsulta categorias(String... categorias) {
        Predicate<Libro> categoriaEspecifica = libro -> Arrays.stream(categorias).allMatch(libro.getCategorias()::contains);
        libroPredicate = (libroPredicate == null) ? categoriaEspecifica : libroPredicate.and(categoriaEspecifica);
        return this;
    }

    @Override
    public IServicioConsulta anyo(String year) {
        Predicate<Libro> anyo = libro -> libro.getFechaPublicacion().contains(year);
        libroPredicate = (libroPredicate == null) ? anyo : libroPredicate.and(anyo);
        return this;
    }


    @Override
    public IServicioConsulta porPeriodoAnyos(String from, String to) {
        Predicate<Libro> pBetweenYears = (libro -> {
            LocalDate fromYear = LocalDate.of(Integer.parseInt(from), 1, 1);
            LocalDate toYear = LocalDate.of(Integer.parseInt(to), 1, 3);
            LocalDate filmYear = LocalDate.of(Integer.parseInt(libro.getAnyo()), 1, 2);
            return filmYear.isAfter(fromYear) && filmYear.isBefore(toYear);
        });
        libroPredicate = (libroPredicate == null) ? pBetweenYears: libroPredicate.and(pBetweenYears);

        return this;
    }

    @Override
    public IServicioConsulta buscarTitulo(String titulo) {
        Predicate<Libro> tituloEncontrado = (libro -> libro.getTitulo().toLowerCase().contains(titulo.toLowerCase()));
        libroPredicate = (libroPredicate == null) ? tituloEncontrado : libroPredicate.and(tituloEncontrado);
        return this;
    }


}
