package com.primerEjemplo.Modelo.Servicio;

import com.primerEjemplo.Modelo.Interfaz.IServicioConsulta;
import com.primerEjemplo.Modelo.Libro;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class Servicio {

    @Autowired
    private ImplementaIntefazServicioGeneral implementaIntefaz;
    @Autowired
    private IServicioConsulta iServicioConsulta ;

    public Servicio()  {

    }

    //CONSULTAS ESPECIFICAS POR CADALIBRO

    public List<String> mostrarCategorias(){
        List<String> categorias;

        categorias = implementaIntefaz.mostrarLibros().stream().map(Libro::getCategorias).
                flatMap(Collection::stream).distinct().sorted().collect(Collectors.toList());

        return categorias;
    }

    public List<Libro> mostrarCategorias(String... generos){
        return iServicioConsulta.cualquierCategoria(generos).ejecutarLibro();
    }

    public List<Libro> mostrarPorCategoria(String... genero){
        return iServicioConsulta.categorias(genero).ejecutarLibro();
    }

    public List<Libro> encontrarPorFecha(String year){
        return iServicioConsulta.anyo(year).ejecutarLibro();
    }

    public List<Libro>  encontrarPorPeriodo(String from, String till){
        return iServicioConsulta.porPeriodoAnyos(from,till).ejecutarLibro();
    }

    public List<Libro> encontrarPorTitulo(String titulo){
        return iServicioConsulta.buscarTitulo(titulo).ejecutarLibro();
    }

    //BUSQUEDA GENERAL
    public List<Libro> mostrarLibros(){
        return (List<Libro>) implementaIntefaz.mostrarLibros();
    }

    public String mostrarResumenLibros(String isbn ){
        return implementaIntefaz.mostrarLibroResumen(isbn);
    }

    public boolean anyadirLibro(Libro libro){
        return implementaIntefaz.anyadirLibro(libro);
    }

    public boolean venderLibro(String isbn){
        return  implementaIntefaz.venderLibro(isbn);
    }

    public Libro buscarLibro(String isbn){
        return  implementaIntefaz.buscarLibro(isbn);
    }


}
